package com.jboss.datagrid.test.jpastore;

import com.jboss.datagrid.test.jpastore.utils.DeploymentBuilder;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.OverProtocol;
import org.jboss.arquillian.testng.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.testng.annotations.AfterClass;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;

import java.lang.reflect.Method;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author vchepeli@redhat.com
 */
public abstract class AbstractJpaITWrapper<T> extends Arquillian {

   @Deployment
   @OverProtocol("Servlet 3.0")
   public static Archive<?> deployment() {
      Archive<?> archive = new DeploymentBuilder()
            .addLibs()
            .addDbDriverLib()
            .addTests(AbstractJpaITWrapper.class)
            .addConfigFiles()
            .build();
      return archive;
   }

   @BeforeClass(alwaysRun = true)
   public void createBeforeClass() throws Throwable {
      Class<?> clazz = null;
      for (clazz = getTestClass().getClass(); clazz != null; clazz = clazz.getSuperclass()) {
         for (Method m : clazz.getDeclaredMethods()) {
            if (m.isAnnotationPresent(BeforeClass.class)) {
               m.setAccessible(true);
               m.invoke(getTestClass(), (Object[]) null);
            }
         }
      }
   }

   @BeforeMethod(alwaysRun = true)
   public void createBeforeMethod() throws Throwable {
      Class<?> clazz = null;
      for (clazz = getTestClass().getClass(); clazz != null; clazz = clazz.getSuperclass()) {
         for (Method m : clazz.getDeclaredMethods()) {
            if (m.isAnnotationPresent(BeforeMethod.class)) {
               m.setAccessible(true);
               m.invoke(getTestClass(), (Object[]) null);
            }
         }
      }
   }

   @AfterClass(alwaysRun = true)
   public void destroy() throws Throwable {
      deregisterJdbcDriver();

      Class<?> clazz = null;
      for (clazz = getTestClass().getClass(); clazz != null; clazz = clazz.getSuperclass()) {
         for (Method m : clazz.getDeclaredMethods()) {
            if (m.isAnnotationPresent(AfterClass.class)) {
               m.setAccessible(true);
               m.invoke(getTestClass(), (Object[]) null);
            }
         }
      }
   }

   @AfterMethod(alwaysRun = true)
   public void clearContent() throws Throwable {
      Class<?> clazz = null;
      for (clazz = getTestClass().getClass(); clazz != null; clazz = clazz.getSuperclass()) {
         for (Method m : clazz.getDeclaredMethods()) {
            if (m.isAnnotationPresent(AfterMethod.class)) {
               m.setAccessible(true);
               m.invoke(getTestClass(), (Object[]) null);
            }
         }
      }
   }

   protected abstract T getTestClass();

   /**
    * Some JDBC drivers (e.g. DB2, Oracle) don't successfully unregister after finishing the deployment, causing
    * accumulation of ClassLoaders and then memory leak => OOME PermGen space. This method forces the unregistration.
    */
   private void deregisterJdbcDriver() {
      Enumeration<Driver> drivers = DriverManager.getDrivers();
      while (drivers.hasMoreElements()) {
         Driver driver = drivers.nextElement();
         try {
            DriverManager.deregisterDriver(driver);
            Logger.getLogger(getTestClass().getClass().getName()).log(Level.INFO, "Deregistering driver " + driver);
         } catch (SQLException e) {
            Logger.getLogger(getTestClass().getClass().getName()).log(Level.SEVERE, "Error deregistering driver " + driver, e);
         }
      }
   }
}
