package com.jboss.datagrid.test.jpastore.utils;

import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.EmptyAsset;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.api.spec.WebArchive;

import java.io.File;

/**
 * @author vchepeli@redhat.com
 */
public class DeploymentBuilder {

   private static String container = System.getProperty("arquillian.launch");
   private WebArchive war;

   public DeploymentBuilder() {
      war = ShrinkWrap.create(WebArchive.class, "test.war");
   }

   public DeploymentBuilder addLibs() {

      // add junit library
      war.addAsLibrary(new File("target/test-libs/junit.jar"));
      // add mockito library
      war.addAsLibrary(new File("target/test-libs/mockito-all.jar"));

      // add infinispan libraries
      war.addAsLibrary(new File("target/test-libs/infinispan-core.jar"));
      war.addAsLibrary(new File("target/test-libs/infinispan-core-tests.jar"));
      war.addAsLibrary(new File("target/test-libs/infinispan-commons.jar"));
      war.addAsLibrary(new File("target/test-libs/infinispan-cachestore-jpa.jar"));

      // add jboss marshalling libs
      war.addAsLibrary(new File("target/test-libs/jboss-marshalling.jar"));
      war.addAsLibrary(new File("target/test-libs/jboss-marshalling-river.jar"));

      if (isWildfly82()) {
         addWildfly82Libs();
         addPersistenceXml("/wildfly/persistence.xml");
      }

      if (isWls12c()) {
         addWls12cLibs();
         addPersistenceXml("/weblogic/persistence.xml");
      }

      // add jta library
      war.addAsLibrary(new File("target/test-libs/jta.jar"));

      return this;
   }

   private void addPersistenceXml(String resource) {
      JavaArchive archive = ShrinkWrap.createFromZipFile(JavaArchive.class, new File("target/test-libs/infinispan-cachestore-jpa-tests.jar"));
      archive.delete("META-INF/persistence.xml");
      archive.addAsManifestResource(getClass().getResource(resource), "persistence.xml");
      war.addAsLibrary(archive.as(JavaArchive.class));
   }

   private void addWildfly82Libs() {
      war.addAsLibrary(new File("target/test-libs/jgroups.jar"));

//      war.addAsManifestResource(getClass().getResource("wildfly/jboss-deployment-structure.xml"), "jboss-deployment-structure.xml");
   }

   private void addWls12cLibs() {
      // add jboss-logging library
      war.addAsLibrary(new File("target/test-libs/jboss-logging.jar"));
      // add jandex library
      war.addAsLibrary(new File("target/test-libs/jandex.jar"));
      // add hibernate libraries
      war.addAsLibrary(new File("target/test-libs/hibernate-commons-annotations.jar"));
      war.addAsLibrary(new File("target/test-libs/hibernate-core.jar"));
      war.addAsLibrary(new File("target/test-libs/hibernate-entitymanager.jar"));
      war.addAsLibrary(new File("target/test-libs/hibernate-jpa-2.0-api.jar"));
      war.addAsLibrary(new File("target/test-libs/hibernate-validator.jar"));
      // add validation library
      war.addAsLibrary(new File("target/test-libs/validation-api.jar"));

      war.addAsWebInfResource(getClass().getResource("/weblogic/weblogic.xml"), "weblogic.xml");
   }

   public DeploymentBuilder addDbDriverLib() {
      war = war.addAsLibraries(new File(System.getProperty("driver.path")));
      return this;
   }

   public DeploymentBuilder addConfigFiles() {
      war.addAsResource(new File("target/test-classes/config/jpa-config.xml"), "config/jpa-config.xml");
      war.addAsWebInfResource(EmptyAsset.INSTANCE, "beans.xml");
      return this;
   }

   public DeploymentBuilder addTests(Class clazz) {
      war = war.addPackages(true, clazz.getPackage());
      return this;
   }

   public WebArchive build() {
      return war;
   }

   public static boolean isWls12c() {
      return container.equalsIgnoreCase("wls12c");
   }

   private boolean isWildfly82() {
      return container.contains("wildfly82");
   }

}